/** \file the header file of QRCodeEncoderDLL.h */

#ifndef QRCODE_FONT_ENCODER_DLL_H
#define QRCODE_FONT_ENCODER_DLL_H

/***************************************************************************
 QRCodeEncoderDLL.h : header file for functions exported from 
 MoroviaQRCodeEncoder5.dll.

 This file is a part of the Morovia QRCode Font & Encoder 5.x package.
 (c)2011 Morovia Corporation. All rights reserved.

 THIS SOURCE FILE IS THE PROPERTY OF MOROVIA CORPORATION AND IS NOT TO
 BE RE-DISTRIBUTED BY ANY MEANS WHATSOEVER WITHOUT THE EXPRESSED
 WRITTEN CONSENT OF MOROVIA.

 THIS SOURCE CODE CAN ONLY BE USED UNDER THE TERMS AND CONDITIONS
 OUTLINED IN THE MOROVIA BARCODE COMPONENT LICENSE AGREEMENT.  MOROVIA
 GRANTS TO YOU (ONE SOFTWARE DEVELOPER) THE LIMITED RIGHT TO USE
 THIS SOFTWARE ON A SINGLE COMPUTER.

 MOROVIA EXPRESSLY DISCLAIMS ANY WARRANTY FOR THE SOFTWARE, DOCUMENTATION
 AND ANYTHING ELSE PROVIDED BY MOROVIA HEREBY AND MOROVIA PROVIDES THE
 SAME IN "AS IS" CONDITION WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS
 OR IMPLIED, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK
 ARISING OUT OF USE OR PERFORMANCE OF THE SOFTWARE AND DOCUMENTATION
 REMAINS WITH YOU.

 CONTACT INFORMATION:
 support@morovia.com
 http://www.morovia.com
********************************************************************************/

/* $LastChangedDate: 2013-06-03 12:33:25 -0400 (Mon, 03 Jun 2013) $ */
/* $Revision: 7349 $ */

#include <windows.h>

struct IStream;
#ifdef __cplusplus
extern "C" {
#endif

/***************************************************************************
 * To use the functions exported from the DLL, include this header file and link
 * to the import lib (MoroviaQRCodeEncoder5.lib). 
 * If you are using Visual C++, add a line in your code
 * #pragma comment(lib, "MoroviaQRCodeFontEncoder5.lib")
 * 
 * You need to add it your library path if this file is not under source directory.
*/

#ifndef MOROVIA_ENCODER_IMAGE_SUPPORT
#define MOROVIA_ENCODER_IMAGE_SUPPORT

/** Image formats supported by the encoder. PNG and BMP are raster
 *  formats, which store color information of individual pixels. 
 *  SVG, EMF and EPS are vector graphics
 *  formats and contains drawing commands instead.
 *  If you are using raster graphic format, one pixel should be mapped to 
 *  one or integral times pixels on the printer. If you are using vector graphics
 *  format, the drawing units should map to integral times pixels on the printer.
 *  */
enum ImageTypeEnum {
  IMAGE_PNG = 0,	/**< PNG (Portable Network Graphics) */
  IMAGE_SVG = 1,	/**< SVG (Scalable Vector Graphics) */
  IMAGE_EMF = 2,	/**< EMF (Windows Enhanced MetaFile) */
  IMAGE_EPS = 3,	/**< EPS (Encapsulated PostScript) */
  IMAGE_BMP = 4		/**< BMP (Windows Bitmap) */
  };
#endif 

/** The \b QRCodeEncode function encodes data and returns a pointer that points 
 *  to encoder result object.
 *  This function encodes the data
 *  according to the parameter specified, and returns a pointer from which you
 *  can retrieve the encoding result.
 *  \param dataToEncode [in] Pointer to a null-terminated string containing the data to be encoded.
 *  You can use tilde codes to encode control characters such as NUL, as well
 *  as advanced features such as ECI and structural append. For more information, 
 *  see [xref input-format]. 
 *  \param versionRequested [in] An integer value that corresponds to the size ID
 *  of the QRCode barcode. QR Code has a total of 40 different sizes, ranging
 *  between 1 and 40.  
 *  If 0 is specified, the 
 *  encoder will select the minimum size that can have the data encoded.
 *  \param ecLevel [in] An integer that sets the error correction level in the barcode.
 *  Valid values are 0, 1, 2 and 3, which correspond to Level L, M, Q and H
 *  respectively. 
 *  \param ppResult A pointer to a pointer that points to the encode result 
 *  created. Use this pointer to discover the actual attributes of the symbol 
 *  created, or create image files. 
 *  \return If the function succeeded, the return value is 0. If the function failed,
 *  it returns the error code. You can call function #QRCodeGetErrorMessage
 *  to obtain the description of the error. 
 */
int __stdcall QRCodeEncode(const char* dataToEncode,
               int versionRequested, int ecLevel, 
               void** ppResult);

/** The \b QRCodeGetParity function calculates the parity value based on the
 *  whole data encoded and split into multiple symbols.
 *  The QR Code structural append features allows data to be split across multiple
 *  symbols ( < 16). A SA capable scanner reads the symbols one  and one, and 
 *  emits the data after all the symbols are read. Symbols belong to the same cluster
 *  share the same parity value, which is calculate based on the full data.
 *  \param dataToEncode [in] Pointer to a null-terminated string containing the data to be encoded.
 *  You must pass the full data string to this parameter. Subsequent calls 
 *  should use ~2[seqno][symbol, parity] plus data that each symbol encodes.
 *  \return The parity value that groups the symbols. It should be between 0 and 255.
 */
int __stdcall QRCodeGetParity(const char* dataToEncode);

               
/** The \b MicroQRCodeEncode function encodes data in Micro QR Code and returns a pointer that points 
 *  to encoder result object.
 *  This function encodes the data
 *  according to the parameter specified, and returns a pointer from which you
 *  can retrieve the encoding result.
 *  \param dataToEncode [in] Pointer to a null-terminated string containing the data to be encoded.
 *  You can use tilde codes to encode control characters such as NUL. For more information, 
 *  see [xref input-format]. 
 *  \param versionRequested [in] An integer value that corresponds to the size ID
 *  of the QRCode barcode. Micro QR Code has a total of 4 sizes: 1, 2, 3 and 4.
 *  If 0 is specified, the 
 *  encoder selects the minimum version that can accomodate the data. 
 *  If value greater than 4 is specified, the encoder uses 4.
 *  \param ecLevel [in] An integer that sets the error correction level in the barcode.
 *  Valid values are 0, 1, 2 and 3, which correspond to Level E, L, M and Q.
 *  respectively. Not all version/eclevel combinations are possible. If ecLevel is
 *  not possible, a lower level is used. If the error correction level is not achieveable
 *  in the version requested, the symbol version is bumped up.
 *  \param ppResult A pointer to a pointer that points to the encode result 
 *  created. Use this pointer to discover the actual attributes of the symbol 
 *  created, or create image files. 
 *  \return If the function succeeded, the return value is 0. If the function failed,
 *  it returns the error code. You can call function #QRCodeGetErrorMessage
 *  to obtain the description of the error. 
 */
int __stdcall MicroQRCodeEncode(const char* dataToEncode,
               int versionRequested, int ecLevel, 
               void** ppResult);
               

/** The \b HIBC_QRCodeEncode function encodes data and returns a pointer that points 
 *  to encoder result object.
 *  This function encodes the data
 *  according to the parameter specified, and returns a pointer from which you
 *  can retrieve the encoding result.
 *  \param dataToEncode [in] Pointer to a null-terminated string containing the data to be encoded.
 *  You can use tilde codes to encode control characters such as NUL, as well
 *  as advanced features such as ECI and structural append. For more information, 
 *  see [xref input-format]. 
 *  \param versionRequested [in] An integer value that corresponds to the size ID
 *  of the QRCode barcode. QR Code has a total of 40 different sizes, ranging
 *  between 1 and 40.  
 *  If 0 is specified, the 
 *  encoder will select the minimum size that can have the data encoded.
 *  \param ecLevel [in] An integer that sets the error correction level in the barcode.
 *  Valid values are 0, 1, 2 and 3, which correspond to Level L, M, Q and H
 *  respectively. 
 *  \param ppResult A pointer to a pointer that points to the encode result 
 *  created. Use this pointer to discover the actual attributes of the symbol 
 *  created, or create image files. 
 *  \return If the function succeeded, the return value is 0. If the function failed,
 *  it returns the error code. You can call function #QRCodeGetErrorMessage
 *  to obtain the description of the error. 
 */
int __stdcall HIBC_QRCodeEncode(const char* dataToEncode,
               int versionRequested, int ecLevel, 
               void** ppResult);
                              
/** The function \b QRCodeEncode2 is an abbreviated version of #QRCodeEncode.
 *  In this function, a pointer is returned instead of status code.
 *  A non-null return value indicates that no error occurred. The pointer
 *  associated resource must be released by calling #DestroyQRCodeEncodeResult.
 *
 *  \param dataToEncode [in] Pointer to a null-terminated string containing the data to be encoded.
 *  You can use tilde codes to encode control characters such as NUL, as well
 *  as advanced features such as ECI. For more information, 
 *  see [xref input-format]. 
 *  \param versionRequested [in] An integer value that corresponds to the size ID
 *  of the QRCode barcode. QR Code has a total of 40 different sizes, ranging
 *  between 1 and 40.  
 *  If 0 is specified, the 
 *  encoder will select the minimum size that can have the data encoded.
 *  \param ecLevel [in] An integer that sets the error correction level in the barcode.
 *  Valid values are 0, 1, 2 and 3, which correspond to Level L, M, Q and H
 *  respectively. 
 *  \param ppResult A pointer to a pointer that points to the encode result 
 *  created. Use this pointer to discover the actual attributes of the symbol 
 *  created, or create image files. 
 *  \return If the function failed, the return value is 0 (NULL). If it succeeds, 
 *  it returns a pointer that points to the result object. You should release
 *  the encoder result object by calling function #DestroyQRCodeEncodeResult.
 */
void* __stdcall QRCodeEncode2(const char* dataToEncode,
               int versionRequested, int ecLevel);


/** The function \b QRCodeEncode2W is used to encode unicode string (UTF-16) into QR Code
 *  in a way compliant with ZXing reader. Read [xref unicode-string-support]
 *  for more information. QR Code does not natively support unicode.
 *  Use of this feature require special support of a reader. 
 *
 *  \param dataToEncode [in] Pointer to a null-terminated UTF16 string 
 *  containing the data to be encoded.
 *  You can use tilde codes to encode control characters such as NUL, as well
 *  as advanced features such as ECI. For more information, 
 *  see [xref input-format]. 
 *  \param versionRequested [in] An integer value that corresponds to the size ID
 *  of the QRCode barcode. QR Code has a total of 40 different sizes, ranging
 *  between 1 and 40.  
 *  If 0 is specified, the 
 *  encoder will select the minimum size that can have the data encoded.
 *  \param ecLevel [in] An integer that sets the error correction level in the barcode.
 *  Valid values are 0, 1, 2 and 3, which correspond to Level L, M, Q and H
 *  respectively. 
 *  \param ppResult A pointer to a pointer that points to the encode result 
 *  created. Use this pointer to discover the actual attributes of the symbol 
 *  created, or create image files. 
 *  \return If the function failed, the return value is 0 (NULL). If it succeeds, 
 *  it returns a pointer that points to the result object. You should release
 *  the encoder result object by calling function #DestroyQRCodeEncodeResult.
 */
void* __stdcall QRCodeEncode2W(const wchar_t* dataToEncode,
               int versionRequested, int ecLevel);

/** The function \b MicroQRCodeEncode2 is an abbreviated version of #MicroQRCodeEncode.
 *  In this function, a pointer is returned instead of status code.
 *  A non-null return value indicates that no error occurred. The pointer
 *  associated resource must be released by calling #DestroyQRCodeEncodeResult.
 *
 *  \param dataToEncode [in] Pointer to a null-terminated string containing the data to be encoded.
 *  You can use tilde codes to encode control characters such as NUL. For more information, 
 *  see [xref input-format]. 
 *  \param versionRequested [in] An integer value that corresponds to the size ID
 *  of the QRCode barcode. Micro QR Code has a total of 4 sizes: 1, 2, 3 and 4.
 *  If 0 is specified, the 
 *  encoder selects the minimum version that can accomodate the data. 
 *  If value greater than 4 is specified, the encoder uses 4.
 *  \param ecLevel [in] An integer that sets the error correction level in the barcode.
 *  Valid values are 0, 1, 2 and 3, which correspond to Level E, L, M and Q.
 *  respectively. Not all version/eclevel combinations are possible. If ecLevel is
 *  not possible, a lower level is used. If the error correction level is not achieveable
 *  in the version requested, the symbol version is bumped up.
 *  \param ppResult A pointer to a pointer that points to the encode result 
 *  created. Use this pointer to discover the actual attributes of the symbol 
 *  created, or create image files. 
 *  \return If the function failed, the return value is 0 (NULL). If it succeeds, 
 *  it returns a pointer that points to the result object. You should release
 *  the encoder result object by calling function #DestroyQRCodeEncodeResult.
 */
void* __stdcall MicroQRCodeEncode2(const char* dataToEncode,
               int versionRequested, int ecLevel);


/** The function \b HIBC_QRCodeEncode2 is an abbreviated version of #QRCodeEncode.
 *  In this function, a pointer is returned instead of status code.
 *  A non-null return value indicates that no error occurred. The pointer
 *  associated resource must be released by calling #DestroyQRCodeEncodeResult.
 *
 *  \param dataToEncode [in] Pointer to a null-terminated string containing the data to be encoded.
 *  You can use tilde codes to encode control characters such as NUL, as well
 *  as advanced features such as ECI. For more information, 
 *  see [xref input-format]. 
 *  \param versionRequested [in] An integer value that corresponds to the size ID
 *  of the QRCode barcode. QR Code has a total of 40 different sizes, ranging
 *  between 1 and 40.  
 *  If 0 is specified, the 
 *  encoder will select the minimum size that can have the data encoded.
 *  \param ecLevel [in] An integer that sets the error correction level in the barcode.
 *  Valid values are 0, 1, 2 and 3, which correspond to Level L, M, Q and H
 *  respectively. 
 *  \param ppResult A pointer to a pointer that points to the encode result 
 *  created. Use this pointer to discover the actual attributes of the symbol 
 *  created, or create image files. 
 *  \return If the function failed, the return value is 0 (NULL). If it succeeds, 
 *  it returns a pointer that points to the result object. You should release
 *  the encoder result object by calling function #DestroyQRCodeEncodeResult.
 */
void* __stdcall HIBC_QRCodeEncode2(const char* dataToEncode,
               int versionRequested, int ecLevel);


/** The \b QRCodeResultGetSizeID function retrieves the size ID of the QRCode
 *  barcode created. 
 *  \param pEncodeResult [in] The pointer that points to the encoding result object,
 *  returned from QRCodeEncode or QRCodeEncode2 functions.
 *  \return The actual version of the barcode. 
 */
int __stdcall QRCodeResultGetVersion(void* pEncodeResult);

/** The \b QRCodeResultGetBarcodeString function retrieves the barcode string
 *  that becomes a QRCode symbol after being formatted with a QRCode font.
 *  When creating barcodes using font-based solution, you first call encoder function
 *  #QRCodeEncode or #QRCodeEncode2 to obtain a pointer that points to the result object.
 *  Then QRCodeResultGetBarcodeString is called to obtain a string that becomes
 *  QRCode symbol after the font is applied. 
 *  If memory is not an issue, allocate a large buffer with 8096 bytes to hold 
 *  the largest QRCode barcode. This size is sufficient for the largest symbol
 *  with carriage return and line feed as line ending.
 *  \param pEncodeResult [in] The pointer that points to the encoding result object,
 *  returned from QRCodeEncode or QRCodeEncode2 functions.
 *  \param buffer [in] The pointer that points to a byte array that receives the
 *  string.
 *  \param maxSize [in,out] Pointer to a variable that specifies the size of the 
 *  buffer pointed to by the buffer parameter, in bytes. When the function returns, 
 *  this variable contains the size of the data copied to buffer. 
 *  \param eol [in] Pointer to a NUL terminated string that will be appended to 
 *  each row in the barcode string.
 *  \return If the function succeeded, it returns the length of the string (excluding
 *  terminating NUL character). If the function fails due to insufficient storage,
 *  it returns 0.
 */
int __stdcall QRCodeResultGetBarcodeString(void* pEncodeResult, char* buffer, 
		unsigned int* maxSize, const char* eol);
		
		
/** The \b QRCodeResultGetBarcodeString2 function returns barcode string without
 *  requiring user to preallocate the buffer.
 *  The encoder result manages the buffer by itself.
 *  Caller should cache the string returned, but not the pointer, as the contents
 *  may change after another call of QRCodeResultBarcodeString2 is made.
 *  \param pEncodeResult [in] The pointer that points to the encoding result object,
 *  returned from QRCodeEncode or QRCodeEncode2 functions.
 *  \param eol [in] Pointer to a NUL terminated string that will be appended to 
 *  each row in the barcode string.
 *  \return Pointer to a NUL terminated string that represents the barcode (barcode string).
 *  If the function failed due to insufficient memory, it returns NULL.
 */
const char* __stdcall QRCodeResultGetBarcodeString2(void* pEncodeResult, const char* eol);
		
		
/** The \b DestroyQRCodeEncodeResult function releases all resources allocated to
 *  the encoder result object. After the object is destroyed, the specific pointer
 *  pResult is no longer valid.
 *  \param pResult [in] Pointer to the encoder result object.
 */
void __stdcall DestroyQRCodeEncodeResult(void* pResult);

/** The \b PaintQRCodeImageRaster function writes the QRCode barcode in raster image format
 *  specified to a disk file. 
 *  Note: the order of the component bytes are R, G and B, which is opposite of 
 *  \b COLORREF type on Windows.
 *  \param pEncodeResult [in] Pointer that points to the encoder result object.
 *  \param wszFilename [in] Pointer to the file name for the image file to be created.
 *  \param pixelsPerModule [in] Number pixels per module with.
 *  \param forecolor [in] Value of the foreground color, in RGB color space. 
 *  \param backcolor [in] Value of the background color, in RGB color space.
 *  \param imageType [in] An integer that indicates the image file format. The 
 *  current version supports two formats: PNG(0) and BMP(4). 
 *  \return If the function failed, it returns the error code. You can call 
 *  function #QRCodeGetErrorMessage
 *  to obtain the description of the error. 
 */
int __stdcall PaintQRCodeImageRaster(void* pEncodeResult, 
				const wchar_t* wszFilename,
				int pixelsPerModule,
				int forecolor,
				int backcolor,
				int imageType);

/** The \b PaintQRCodeImageRaster function writes the QRCode barcode in raster image format
 *  specified to IStream object. 
 *  Note: the order of the component bytes are R, G and B, which is opposite of 
 *  \b COLORREF type on Windows.
 *  \param pEncodeResult [in] Pointer that points to the encoder result object.
 *  \param ostream [in] Pointer to an IStream object. 
 *  \param pixelsPerModule [in] Number pixels per module with.
 *  \param forecolor [in] Value of the foreground color, in RGB color space. 
 *  \param backcolor [in] Value of the background color, in RGB color space.
 *  \param imageType [in] An integer that indicates the image file format. The 
 *  current version supports two formats: PNG(0) and BMP(4). 
 *  \return If the function failed, it returns the error code. You can call 
 *  function #QRCodeGetErrorMessage
 *  to obtain the description of the error. 
 */
int __stdcall PaintQRCodeImageRaster2(void* pEncodeResult, 
				IStream* ostream,
				int pixelsPerModule,
				int forecolor,
				int backcolor,
				int imageType);


/** The \b QRCodeGetErrorMessage function retrieves a string that describes the error.
 *  The string returned is a read only string. User should not modify the string directly.
 *  \param errorno [in] The error number.
 *  \return a read only string that describes the error.
 */
const char* __stdcall QRCodeGetErrorMessage(int errorno);

/** The \b PaintQRCodeImageVector function writes the QRCode barcode in vector image format
 *  specified to a disk file. 
 *  Note: the order of the component bytes are R, G and B, which is opposite of 
 *  \b COLORREF type on Windows.
 *  \param pEncodeResult [in] Pointer that points to the encoder result object.
 *  \param wszFilename [in] Pointer to the file name for the image file to be created.
 *  \param module_width_hm [in] Module width (X dimension), in the unit of high metric. 
 *  1 unit high metric = 1/1000 cm.
 *  \param target_dpi [in] The resolution of the target printer. 
 *  \param forecolor [in] Value of the foreground color, in RGB color space. 
 *  \param backcolor [in] Value of the background color, in RGB color space.
 *  \param imageType [in] An integer that indicates the image file format. The 
 *  current version supports three formats: SVG(1), EMF(2) and EPS(3). 
 *  \return If the function failed, it returns the error code. You can call 
 *  function #QRCodeGetErrorMessage
 *  to obtain the description of the error. 
 */
int __stdcall PaintQRCodeImageVector(void* pEncodeResult, 
				const wchar_t* wszFilename,
				int module_width_hm,
				int target_dpi,
				int forecolor,
				int backcolor,
				int imageType);

/** The \b PaintQRCodeImageVector function writes the QRCode barcode in vector image format
 *  specified to an IStream object. 
 *  Note: the order of the component bytes are R, G and B, which is opposite of 
 *  \b COLORREF type on Windows.
 *  \param pEncodeResult [in] Pointer that points to the encoder result object.
 *  \param ostream [in] Pointer to an IStream object. 
 *  \param module_width_hm [in] Module width (X dimension), in the unit of high metric. 
 *  1 unit high metric = 1/1000 cm.
 *  \param target_dpi [in] The resolution of the target printer. 
 *  \param forecolor [in] Value of the foreground color, in RGB color space. 
 *  \param backcolor [in] Value of the background color, in RGB color space.
 *  \param imageType [in] An integer that indicates the image file format. The 
 *  current version supports three formats: SVG(1), EMF(2) and EPS(3). 
 *  \return If the function failed, it returns the error code. You can call 
 *  function #QRCodeGetErrorMessage
 *  to obtain the description of the error. 
 */
int __stdcall PaintQRCodeImageVector2(void* pEncodeResult, 
				IStream* ostream,
				int module_width_hm,
				int target_dpi,
				int forecolor,
				int backcolor,
				int imageType);
 
/** The \b PaintQRCodeImageClipboard function Creates a Windows Enhanced Meta File 
 * object and place it into the clipboard. 
 *  \param pEncodeResult [in] Pointer that points to the encoder result object.
 *  \param module_width_hm [in] Module width (X dimension), in the unit of high metric. 
 *  1 unit high metric = 1/1000 cm.
 *  \param target_dpi [in] The resolution of the target printer. 
 *  \param forecolor [in] Value of the foreground color, in RGB color space. 
 *  \param backcolor [in] Value of the background color, in RGB color space.
 *  \return If the function failed, it returns the error code. You can call 
 *  function #QRCodeGetErrorMessage
 *  to obtain the description of the error. 
 */
int __stdcall PaintQRCodeImageClipboard(void* pEncodeResult, 
	int module_width_hm,
	int target_dpi,
	int forecolor,
	int backcolor);
	
/** The \b PaintQRCodeImageClipboard function Creates a Windows Enhanced Meta File 
 *  object and returns the handle. The caller should destroy this handle after using.
 *  \param pEncodeResult [in] Pointer that points to the encoder result object.
 *  \param module_width_hm [in] Module width (X dimension), in the unit of high metric. 
 *  1 unit high metric = 1/1000 cm.
 *  \param target_dpi [in] The resolution of the target printer. 
 *  \param forecolor [in] Value of the foreground color, in RGB color space. 
 *  \param backcolor [in] Value of the background color, in RGB color space.
 *  \return If the function failed, it returns the error code. You can call 
 *  function #QRCodeGetErrorMessage
 *  to obtain the description of the error. 
 */
	
HENHMETAFILE __stdcall PaintQRCodeImageEMF(void* pEncodeResult,
		int module_len_hm,
		int target_dpi,
		int forecolor,
		int backcolor,
		int imageType);
	
#ifdef __cplusplus
}
#endif


#endif /* QRCode_FONT_ENCODER_DLL_H */